#/usr/bin/python3

import libtmux
import time
import re
import subprocess

def get_start_timestamp(service: str) -> int:
    output = subprocess.run(
            ['systemctl', 'show', '--property', 'ExecMainStartTimestampMonotonic', service],
            capture_output=True
        ).stdout
    return int(output.split(b'=')[-1])

def get_session(server: libtmux.Server, name) -> libtmux.Session:
    return server.new_session(session_name=name, attach=False, window_command='bash -l', x=132, y=80)

def test_no_interaction(cli: str, session: libtmux.Session):
    dbus_start = get_start_timestamp('dbus.service')
    cron_start = get_start_timestamp('cron.service')
    pane = session.attached_pane
    assert pane
    pane.send_keys(cli)
    time.sleep(15) # This ought to be enough for that operation
    lines = pane.capture_pane()
    assert not isinstance(lines, str)
    anchor, restarted, deferred = None, None, None
    print()
    print("TMUX SCREEN:")
    print("\n".join(lines))
    print()
    for i, line in enumerate(lines):
        if line.startswith('Restarting services...'):
            anchor = i
            continue
        if line.startswith('Service restarts being deferred:'):
            assert anchor is not None
            restarted = ' '.join(l.strip() for l in lines[anchor+1:i])
            continue
        if restarted is not None and line.strip():
            deferred = '\n'.join(l.strip() for l in lines[anchor+1:i])

    # Just quick check that needrestart retstarts things (using cron as our canary)
    assert restarted is not None and re.match(r'systemctl restart .*cron\.service', restarted)
    assert cron_start != get_start_timestamp('cron.service')

    # Also check that some are explictly *not* restarted, using dbus as our example
    assert deferred is not None and 'dbus.service' in deferred
    assert dbus_start == get_start_timestamp('dbus.service')

def test_default_case(server: libtmux.Server):
    print()
    print("default case")
    print()
    session = get_session(server, 'needrestart-default')
    try:
        test_no_interaction('LANG=C DEBIAN_FRONTEND=dialog apt reinstall libc6', session)
    finally:
        session.kill()

def test_noninteractive(server: libtmux.Server):
    print()
    print("non interactive")
    print()
    session = get_session(server, 'needrestart-default')
    try:
        test_no_interaction('true | LANG=C DEBIAN_FRONTEND=noninteractive apt reinstall libc6 2>&1 | cat', session)
    finally:
        session.kill()

def test_debconf(server: libtmux.Server):
    print()
    print("debconf")
    print()
    session = get_session(server, 'needrestart-medium')
    try:
        dbus_start = get_start_timestamp('dbus.service')
        cron_start = get_start_timestamp('cron.service')
        pane = session.attached_pane
        assert pane
        # autopkgtest environment sets DEBIAN_FRONTEND, so we need to explicitly set it back
        pane.send_keys('LANG=C DEBIAN_FRONTEND=dialog NEEDRESTART_UI=NeedRestart::UI::Debconf apt reinstall libc6')
        time.sleep(15) # This ought to be enough for that operation IMHO

        # At this point we should be in the interactive needrestart prompt
        lines = pane.capture_pane()
        assert not isinstance(lines, str)
        print()
        print("TMUX SCREEN:")
        print("\n".join(lines))
        print()
        assert 'Which services should be restarted?' in '\n'.join(lines)

        pane.send_keys('\t') # Just validates the default prompt

        time.sleep(5) # Just to make sure things settle.
        lines = pane.capture_pane()
        assert not isinstance(lines, str)
        print()
        print("TMUX SCREEN:")
        print("\n".join(lines))
        print()
        anchor, restarted, deferred = None, None, None
        for i, line in enumerate(lines):
            if line.startswith('Restarting services...'):
                anchor = i
                continue
            if line.startswith('Service restarts being deferred:'):
                assert anchor is not None
                restarted = ' '.join(l.strip() for l in lines[anchor+1:i])
                continue
            if restarted is not None and line.strip():
                deferred = '\n'.join(l.strip() for l in lines[anchor+1:i])

        # Just quick check that needrestart retstarts things (using cron as our canary)
        assert restarted is not None and re.match(r'systemctl restart .*cron\.service', restarted)
        assert cron_start != get_start_timestamp('cron.service')

        # Also check that some are explictly *not* restarted, using dbus as our example
        assert deferred is not None and 'dbus.service' in deferred
        assert dbus_start == get_start_timestamp('dbus.service')
    finally:
        session.kill()

if __name__ == '__main__':
    SOCKET_NAME='needrestart-tmux'
    server = libtmux.Server(socket_name=SOCKET_NAME)
    try:
        test_default_case(server)
        test_noninteractive(server)
        test_debconf(server)
    finally:
        server.kill()

